@@ -12,10 +12,9 @@
 
 #include "gmp_main.h"
 #include "prime_iterator.h"
-#include "small_factor.h"
-#include "simpqs.h"
-#include "ecm.h"
-#define _GMP_ECM_FACTOR _GMP_ecm_factor_projective
+#include "bls75.h"
+#include "ecpp.h"
+#include "utility.h"
 
 static int _verbose = 0;
 void _GMP_set_verbose(int v) { _verbose = v; }
@@ -60,6 +59,32 @@ static INLINE int _GMP_miller_rabin_ui(mpz_t n, UV base)
   return rval;
 }
 
+int _GMP_miller_rabin_random(mpz_t n, UV numbases)
+{
+  gmp_randstate_t* p_randstate = _GMP_get_randstate();
+  mpz_t base;
+  UV i;
+
+  /* We could just use mpz_probab_prime_p and call it a day. */
+
+  /* Make sure we can make proper random bases */
+  if (mpz_cmp_ui(n, 2) < 0) return 0;  /* below 2 is composite */
+  if (mpz_cmp_ui(n, 4) < 0) return 1;  /* 2 and 3 are prime */
+  if (mpz_even_p(n))        return 0;  /* multiple of 2 is composite */
+
+  mpz_init(base);
+  for (i = 0; i < numbases; i++) {
+    /* select a random base between 2 and n-2 (we're lazy and use n-1) */
+    do {
+      mpz_urandomm(base, *p_randstate, n);
+    } while (mpz_cmp(base, n) >= 0 || mpz_cmp_ui(base, 1) <= 0);
+    if (_GMP_miller_rabin(n, base) == 0)
+      break;
+  }
+  mpz_clear(base);
+  return (i >= numbases);
+}
+
 int _GMP_miller_rabin(mpz_t n, mpz_t a)
 {
   mpz_t nminus1, d, x;
@@ -74,22 +99,22 @@ int _GMP_miller_rabin(mpz_t n, mpz_t a)
   }
   mpz_init_set(nminus1, n);
   mpz_sub_ui(nminus1, nminus1, 1);
-  mpz_init_set(d, nminus1);
+  mpz_init_set(x, a);
 
-  s = mpz_scan1(d, 0);
-  mpz_tdiv_q_2exp(d, d, s);
-
-  /* Handle large and small bases.  TODO: don't modify their input a */
-  if (mpz_cmp(a, n) >= 0)
-    mpz_mod(a, a, n);
-  if ( (mpz_cmp_ui(a, 1) <= 0) || (mpz_cmp(a, nminus1) >= 0) ) {
+  /* Handle large and small bases.  Use x so we don't modify their input a. */
+  if (mpz_cmp(x, n) >= 0)
+    mpz_mod(x, x, n);
+  if ( (mpz_cmp_ui(x, 1) <= 0) || (mpz_cmp(x, nminus1) >= 0) ) {
     mpz_clear(nminus1);
-    mpz_clear(d);
+    mpz_clear(x);
     return 1;
   }
 
-  mpz_init(x);
-  mpz_powm(x, a, d, n);
+  mpz_init_set(d, nminus1);
+  s = mpz_scan1(d, 0);
+  mpz_tdiv_q_2exp(d, d, s);
+
+  mpz_powm(x, x, d, n);
   mpz_clear(d); /* done with a and d */
   rval = 0;
   if (!mpz_cmp_ui(x, 1) || !mpz_cmp(x, nminus1)) {
@@ -251,6 +276,26 @@ UV _GMP_trial_factor(mpz_t n, UV from_n, UV to_n)
   return 0;
 }
 
+/*
+ * is_prob_prime      BPSW -- fast, no known counterexamples
+ * is_prime           is_prob_prime + a little extra
+ * is_provable_prime  really prove it, which could take a very long time
+ *
+ * They're all identical for numbers <= 2^64.
+ *
+ * For 128-bit primes:
+ *                      is_prob_prime           2 uS
+ *                      is_prime              140 uS  (+4 MR, 50% proven)
+ *                      is_provable_prime     400 uS
+ * For 256-bit primes:
+ *                      is_prob_prime           2 uS
+ *                      is_prime                2 uS  (+4 MR)
+ *                      is_provable_prime    6600 uS
+ *
+ * The extra M-R tests in is_prime start actually costing something after
+ * 1000 bits or so.  Primality proving will get quite a bit slower as the
+ * number of bits increases.
+ */
 
 int _GMP_is_prob_prime(mpz_t n)
 {
@@ -276,146 +321,64 @@ int _GMP_is_prob_prime(mpz_t n)
 
 int _GMP_is_prime(mpz_t n)
 {
-  int prob_prime;
+  int prob_prime = _GMP_is_prob_prime(n);
 
-  prob_prime = _GMP_is_prob_prime(n);
-  if (prob_prime != 1)
-    return prob_prime;
-  /*
-   * BPSW indicated this is a prime, so we're pretty darn sure it is.  For
-   * smaller numbers, try a quick proof using the Pocklington or BLS test.
-   * We only do this for reasonably small numbers, and we're rather
-   * half-hearted about the test, as it requires partially factoring n-1,
-   * which may be easy or may be very hard.  They might not care at all
-   * about this, so don't spend too much time.
-   *
-   * The idea is that they can call:
-   *    is_prob_prime      BPSW
-   *    is_prime           is_prob_prime + a little extra
-   *    is_provable_prime  really prove it, which could take hours or days
-   *
-   * If we had better large-number factoring, we could extend the reasonable
-   * range for this test.  In the long run, ECPP is the better method.  What
-   * Pocklington/BLS gives us is a quite fast and very easy (in code) way to
-   * prove primality for a lot of numbers in the 2^64 - 2^200 bit range (up
-   * to about 50 digits).
-   */
-  if (mpz_sizeinbase(n, 2) <= 200) {
-    prob_prime = _GMP_primality_bls(n, 1);
-    if (prob_prime < 0)
-      return 0;
-    if (prob_prime > 0)
-      return 2;
-  }
-  return 1;
+  /* We're pretty sure n is a prime since it passed the BPSW test.  Try
+   * 10 random M-R bases to give some extra assurance. */
+  if (prob_prime == 1)
+    prob_prime = _GMP_miller_rabin_random(n, 4);
+
+  /* For small numbers, try a quick BLS75 n-1 proof. */
+  if (prob_prime == 1 && mpz_sizeinbase(n, 2) <= 200)
+    prob_prime = _GMP_primality_bls_nm1(n, 2 /* effort */, 0 /* proof */);
+
+  return prob_prime;
 }
 
-int _GMP_is_provable_prime(mpz_t n)
+int _GMP_is_provable_prime(mpz_t n, char** prooftext)
 {
-  int prob_prime;
+  int prob_prime = _GMP_is_prob_prime(n);
 
-  prob_prime = _GMP_is_prob_prime(n);
-  if (prob_prime != 1)
-    return prob_prime;
+  /* The primality proving algorithms tend to be VERY slow for composites,
+   * so run a few more MR tests. */
+  if (prob_prime == 1)
+    prob_prime = _GMP_miller_rabin_random(n, 4);
 
-  /* Miller-Rabin tests are fast, and the primality proving methods are
-   * really, really, really slow for composites.  So run a few more MR tests
-   * just in case.  These bases aren't special.
+  /* We can choose a primality proving algorithm:
+   *   AKS    _GMP_is_aks_prime       really slow, don't bother
+   *   N-1    _GMP_primality_bls_nm1  small or special numbers
+   *   ECPP   _GMP_ecpp               fastest in general
    */
-  if (    (_GMP_miller_rabin_ui(n, 325) == 0)
-       || (_GMP_miller_rabin_ui(n, 131) == 0)
-       || (_GMP_miller_rabin_ui(n, 223) == 0)
-       || (_GMP_miller_rabin_ui(n, 887) == 0)
-     )
-    return 0;
 
-  prob_prime = _GMP_primality_bls(n, 0);
-  if (prob_prime < 0)
-    return 0;
-  if (prob_prime > 0)
-    return 2;
-  return 1;
+  /* Give n-1 a small go */
+  if (prob_prime == 1)
+    prob_prime = _GMP_primality_bls_nm1(n, 2, prooftext);
+
+  /* ECPP */
+  if (prob_prime == 1)
+    prob_prime = _GMP_ecpp(n, prooftext);
+
+  return prob_prime;
 }
 
+#if 0
+  /* This would be useful for a Bernstein AKS variant */
+static UV largest_factor(UV n) {
+  UV p = 2;
+  PRIME_ITERATOR(iter);
+  while (n >= p*p && !prime_iterator_isprime(&iter, n)) {
+    while ( (n % p) == 0  &&  n >= p*p ) { n /= p; }
+    p = prime_iterator_next(&iter);
+  }
+  prime_iterator_destroy(&iter);
+  return n;
+}
+#endif
+
 /*****************************************************************************/
 /*          AKS.    This implementation is quite slow, but useful to have.   */
 
-static UV order(UV r, mpz_t n, UV limit) {
-  UV j;
-  mpz_t t;
-
-  mpz_init_set_ui(t, 1);
-  for (j = 1; j <= limit; j++) {
-    mpz_mul(t, t, n);
-    mpz_mod_ui(t, t, r);
-    if (!mpz_cmp_ui(t, 1))
-      break;
-  }
-  mpz_clear(t);
-  return j;
-}
-
-static void poly_mod_mul(mpz_t* px, mpz_t* py, mpz_t* ptmp, UV r, mpz_t mod)
-{
-  UV i, j, prindex;
-
-  for (i = 0; i < r; i++)
-    mpz_set_ui(ptmp[i], 0);
-  for (i = 0; i < r; i++) {
-    if (!mpz_sgn(px[i])) continue;
-    for (j = 0; j < r; j++) {
-      if (!mpz_sgn(py[j])) continue;
-      prindex = (i+j) % r;
-      mpz_addmul( ptmp[prindex], px[i], py[j] );
-    }
-  }
-  /* Put ptmp into px and mod n */
-  for (i = 0; i < r; i++)
-    mpz_mod(px[i], ptmp[i], mod);
-}
-static void poly_mod_sqr(mpz_t* px, mpz_t* ptmp, UV r, mpz_t mod)
-{
-  UV i, d, s;
-  UV degree = r-1;
-
-  for (i = 0; i < r; i++)
-    mpz_set_ui(ptmp[i], 0);
-  for (d = 0; d <= 2*degree; d++) {
-    UV prindex = d % r;
-    for (s = (d <= degree) ? 0 : d-degree; s <= (d/2); s++) {
-      if (s*2 == d) {
-        mpz_addmul( ptmp[prindex], px[s], px[s] );
-      } else {
-        mpz_addmul( ptmp[prindex], px[s], px[d-s] );
-        mpz_addmul( ptmp[prindex], px[s], px[d-s] );
-      }
-    }
-  }
-  /* Put ptmp into px and mod n */
-  for (i = 0; i < r; i++)
-    mpz_mod(px[i], ptmp[i], mod);
-}
-
-static void poly_mod_pow(mpz_t *pres, mpz_t *pn, mpz_t *ptmp, mpz_t power, UV r, mpz_t mod)
-{
-  UV i;
-  mpz_t mpow;
-
-  for (i = 0; i < r; i++)
-    mpz_set_ui(pres[i], 0);
-  mpz_set_ui(pres[0], 1);
-
-  mpz_init_set(mpow, power);
-
-  while (mpz_cmp_ui(mpow, 0) > 0) {
-    if (mpz_odd_p(mpow))            poly_mod_mul(pres, pn, ptmp, r, mod);
-    mpz_tdiv_q_2exp(mpow, mpow, 1);
-    if (mpz_cmp_ui(mpow, 0) > 0)    poly_mod_sqr(pn, ptmp, r, mod);
-  }
-  mpz_clear(mpow);
-}
-
-static int test_anr(UV a, mpz_t n, UV r, mpz_t* px, mpz_t* py, mpz_t* ptmp)
+static int test_anr(UV a, mpz_t n, UV r, mpz_t* px, mpz_t* py)
 {
   int retval = 1;
   UV i, n_mod_r;
@@ -428,7 +391,7 @@ static int test_anr(UV a, mpz_t n, UV r, mpz_t* px, mpz_t* py, mpz_t* ptmp)
   mpz_set_ui(px[0], a);
   mpz_set_ui(px[1], 1);
 
-  poly_mod_pow(py, px, ptmp, n, r, n);
+  poly_mod_pow(py, px, n, r, n);
 
   mpz_init(t);
   n_mod_r = mpz_mod_ui(t, n, r);
@@ -448,10 +411,11 @@ static int test_anr(UV a, mpz_t n, UV r, mpz_t* px, mpz_t* py, mpz_t* ptmp)
 int _GMP_is_aks_prime(mpz_t n)
 {
   mpz_t sqrtn;
-  mpz_t *px, *py, *pt;
+  mpz_t *px, *py;
   int retval;
-  UV i, log2n, limit, rlimit, r, a;
-  PRIME_ITERATOR(iter);
+  UV i, limit, rlimit, r, a;
+  double log2n;
+  /* PRIME_ITERATOR(iter); */
 
   if (mpz_cmp_ui(n, 4) < 0) {
     return (mpz_cmp_ui(n, 1) <= 0) ? 0 : 1;
@@ -463,9 +427,20 @@ int _GMP_is_aks_prime(mpz_t n)
 
   mpz_init(sqrtn);
   mpz_sqrt(sqrtn, n);
-  /* limit should be floor(log2(n) * log2(n)).  This overcalculates it. */
-  log2n = mpz_sizeinbase(n, 2);  /* ceil log2(n) */
-  limit = log2n * log2n;
+  /* limit should be floor( log2(n) ** 2 ).  The simple GMP solution is
+   * to get ceil(log2(n)) viz mpz_sizeinbase(n,2) and square, but that
+   * overcalculates by a fair amount.  We'll calculate float log2n as:
+   *   ceil(log2(n**k)) / k  [mpz_sizeinbase(n,2) <=> ceil(log2(n))]
+   * which gives us a value that slightly overestimates log2(n).
+   */
+  {
+    mpz_t t;
+    mpz_init(t);
+    mpz_pow_ui(t, n, 32);
+    log2n = ((double) mpz_sizeinbase(t, 2) + 0.000001) / 32.0;
+    limit = (UV) floor( log2n * log2n );
+    mpz_clear(t);
+  }
 
   if (_verbose>1) gmp_printf("# AKS checking order_r(%Zd) to %lu\n", n, (unsigned long) limit);
 
@@ -473,21 +448,22 @@ int _GMP_is_aks_prime(mpz_t n)
    * but would typically work for 100,000+ digits (r ~ log^3n).  This code is
    * far too slow to matter either way. */
 
-  for (r = 2; mpz_cmp_ui(n, r) >= 0; r = prime_iterator_next(&iter)) {
+  /* Note, since order_r(n) can never be > r, we should start r at limit+1 */
+  for (r = limit+1; mpz_cmp_ui(n, r) >= 0; r++) {
     if (mpz_divisible_ui_p(n, r)) {   /* r divides n.  composite. */
-      prime_iterator_destroy(&iter);
+      /* prime_iterator_destroy(&iter); */
       mpz_clear(sqrtn);
       return 0;
     }
     if (mpz_cmp_ui(sqrtn, r) < 0) {  /* no r <= sqrtn divides n.  prime. */
-      prime_iterator_destroy(&iter);
+      /* prime_iterator_destroy(&iter); */
       mpz_clear(sqrtn);
       return 1;
     }
-    if (order(r, n, limit) > limit)
+    if (mpz_order_ui(r, n, limit) > limit)
       break;
   }
-  prime_iterator_destroy(&iter);
+  /* prime_iterator_destroy(&iter); */
   mpz_clear(sqrtn);
 
   if (mpz_cmp_ui(n, r) <= 0) {
@@ -501,18 +477,16 @@ int _GMP_is_aks_prime(mpz_t n)
   /* Create the three polynomials we will use */
   New(0, px, r, mpz_t);
   New(0, py, r, mpz_t);
-  New(0, pt, r, mpz_t);
-  if ( !px || !py || !pt )
+  if ( !px || !py )
     croak("allocation failure\n");
   for (i = 0; i < r; i++) {
     mpz_init(px[i]);
     mpz_init(py[i]);
-    mpz_init(pt[i]);
   }
 
   retval = 1;
   for (a = 1; a <= rlimit; a++) {
-    if (! test_anr(a, n, r, px, py, pt) ) {
+    if (! test_anr(a, n, r, px, py) ) {
       retval = 0;
       break;
     }
@@ -524,11 +498,9 @@ int _GMP_is_aks_prime(mpz_t n)
   for (i = 0; i < r; i++) {
     mpz_clear(px[i]);
     mpz_clear(py[i]);
-    mpz_clear(pt[i]);
   }
   Safefree(px);
   Safefree(py);
-  Safefree(pt);
 
   return retval;
 }
@@ -536,420 +508,6 @@ int _GMP_is_aks_prime(mpz_t n)
 /*****************************************************************************/
 
 
-/*
- * Lucas (1876): Given a completely factored n-1, if there exists an a s.t.
- *     a^(n-1) % n = 1
- *     a^((n-1/f) % n != 1 for ALL factors f of n-1
- * then n is prime.
- *
- * PPBLS:, given n-1 = A*B, A > sqrt(n), if we can find an a s.t.
- *     a^A % n = 1
- *     gcd(a^(A/f)-1,n) = 1 for ALL factors f of A
- * then n is prime.
- *
- * Generalized Pocklington: given n-1 = A*B, gcd(A,B)=1, A > sqrt(n), then if
- *     for each each factor f of A, there exists an a (1 < a < n-1) s.t.
- *         a^(n-1) % n = 1
- *         gcd(a^((n-1)/f)-1,n) = 1
- * then n is prime.
- *
- * BLS: given n-1 = A*B, factored A, s=B/2A r=B mod (2A), and an a, then if:
- *   - A is even, B is odd, and AB=n-1 (all implied by n = odd and the above),
- *   - (A+1) * (2*A*A + (r-1) * A + 1) > n
- *   - a^(n-1) % n = 1
- *   - gcd(a^((n-1)/f)-1,n) = 1  for ALL factors f of A
- * then:
- *     if s = 0 or r*r - 8*s is not a perfect square
- *         n is prime
- *     else
- *         n is composite
- *
- * The generalized Pocklington test is also sometimes known as the
- * Pocklington-Lehmer test.  It's definitely an improvement over Lucas
- * since we only have to find factors up to sqrt(n), _and_ we can choose
- * a different 'a' value for each factor.  This is corollary 1 from BLS75.
- *
- * BLS is the Brillhart-Lehmer-Selfridge 1975 theorem 5 (see link below).
- * We can factor even less of n, and the test lets us kick out some
- * composites early, without having to test n-3 different 'a' values.
- *
- * Once we've found the factors of n-1 (or enough of them), verification
- * usually happens really fast.  a=2 works for most, and few seem to require
- * more than ~ log2(n).  However all but BLS75 require testing all integers
- * 1 < a < n-1 before answering in the negative, which is impractical.
- *
- *
- * AKS is not too hard to implement, but it's impractically slow.
- *
- * ECPP is very fast and definitely the best method for most numbers.
- *
- * BLS75:  http://www.ams.org/journals/mcom/1975-29-130/S0025-5718-1975-0384673-1/S0025-5718-1975-0384673-1.pdf
- *
- */
-
-/* For these primality functions:
- *   -1   n is definitely composite
- *    1   n is definitely prime
- *    0   we can't decide
- *
- * You really should run is_prob_prime on n first, so we only have to run
- * these tests on numbers that are very probably prime.
- */
-
-/* FIXME:
- *   (1) too much repetitious overhead code in these
- *   (2) way too much copy/paste between Pocklington and BLS
- */
-#define PRIM_STACK_SIZE 128
-
-#define primality_handle_factor(f, primality_func, factor_prob) \
-  { \
-    int f_prob_prime = _GMP_is_prob_prime(f); \
-    if ( (f_prob_prime == 1) && (primality_func(f, do_quick)) ) \
-      f_prob_prime = 2; \
-    if (f_prob_prime == 2) { \
-      if (fsp >= PRIM_STACK_SIZE) { success = 0; } \
-      else                        { mpz_init_set(fstack[fsp++], f); } \
-      while (mpz_divisible_p(B, f)) { \
-        mpz_mul(A, A, f); \
-        mpz_divexact(B, B, f); \
-      } \
-    } else if ( (f_prob_prime == 0) || (factor_prob) ) { \
-      if (msp >= PRIM_STACK_SIZE) { success = 0; } \
-      else                        { mpz_init_set(mstack[msp++], f); } \
-    } \
-  }
-
-#if 0
-int _GMP_primality_pocklington(mpz_t n, int do_quick)
-{
-  mpz_t nm1, A, B, sqrtn, t, m, f;
-  mpz_t mstack[PRIM_STACK_SIZE];
-  mpz_t fstack[PRIM_STACK_SIZE];
-  int msp = 0;
-  int fsp = 0;
-  int success = 1;
-
-  mpz_init(nm1);
-  mpz_sub_ui(nm1, n, 1);
-  mpz_init_set_ui(A, 1);
-  mpz_init_set(B, nm1);
-  mpz_init(sqrtn);
-  mpz_sqrt(sqrtn, n);
-  mpz_init(m);
-  mpz_init(f);
-  mpz_init(t);
-
-  { /* Pull small factors out */
-    UV tf = 2;
-    while ( (tf = _GMP_trial_factor(B, tf, 1000)) != 0 ) {
-      if (fsp >= PRIM_STACK_SIZE) { success = 0; break; }
-      mpz_init_set_ui(fstack[fsp++], tf);
-      while (mpz_divisible_ui_p(B, tf)) {
-        mpz_mul_ui(A, A, tf);
-        mpz_divexact_ui(B, B, tf);
-      }
-      tf++;
-    }
-  }
-
-  if (success) {
-    mpz_set(f, B);
-    primality_handle_factor(f, _GMP_primality_pocklington, 1);
-  }
-
-  while (success) {
-    mpz_gcd(t, A, B);
-    if ( (mpz_cmp(A, sqrtn) > 0) && (mpz_cmp_ui(t, 1) == 0) )
-      break;
-    success = 0;
-    /* If the stack is empty, we have failed. */
-    if (msp == 0)
-      break;
-    /* pop a component off the stack */
-    mpz_set(m, mstack[--msp]); mpz_clear(mstack[msp]);
-
-    /* Try to factor it without trying too hard */
-    if (!success)  success = _GMP_power_factor(m, f);
-    if (do_quick) {
-      UV log2m = mpz_sizeinbase(m, 2);
-      UV rounds = (log2m <= 64) ? 300000 : 300000 / (log2m-63);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, rounds);
-    } else {
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 5, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 7, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f,11, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f,13, 64*1024);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 1, 16*1024*1024);
-      if (!success)  success = _GMP_ECM_FACTOR   (m, f, 12500, 4);
-      if (!success)  success = _GMP_ECM_FACTOR   (m, f, 3125000, 10);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, 256*1024*1024);
-      if (!success)  success = _GMP_ECM_FACTOR   (m, f, 400000000, 200);
-    }
-    /* If we couldn't factor m and the stack is empty, we've failed. */
-    if ( (!success) && (msp == 0) )
-      break;
-    /* Put the two factors f and m/f into the stacks */
-    primality_handle_factor(f, _GMP_primality_pocklington, 0);
-    mpz_divexact(f, m, f);
-    primality_handle_factor(f, _GMP_primality_pocklington, 0);
-  }
-  if (success) {
-    int pcount, a;
-    int const alimit = do_quick ? 200 : 10000;
-    mpz_t p, ap;
-
-    mpz_init(p);
-    mpz_init(ap);
-
-    for (pcount = 0; success && pcount < fsp; pcount++) {
-      mpz_set(p, fstack[pcount]);
-      success = 0;
-      for (a = 2; !success && a <= alimit; a = next_small_prime(a)) {
-        mpz_set_ui(ap, a);
-        /* Does a^(n-1) % n = 1 ? */
-        mpz_powm(t, ap, nm1, n);
-        if (mpz_cmp_ui(t, 1) != 0)
-          continue;
-        /* Does gcd(a^((n-1)/f)-1,n) = 1 ? */
-        mpz_divexact(B, nm1, p);
-        mpz_powm(t, ap, B, n);
-        mpz_sub_ui(t, t, 1);
-        mpz_gcd(t, t, n);
-        if (mpz_cmp_ui(t, 1) != 0)
-          continue;
-        success = 1;   /* We found an a for this p */
-      }
-    }
-    mpz_clear(p);
-    mpz_clear(ap);
-  }
-  while (msp-- > 0) {
-    mpz_clear(mstack[msp]);
-  }
-  while (fsp-- > 0) {
-    mpz_clear(fstack[fsp]);
-  }
-  mpz_clear(nm1);
-  mpz_clear(A);
-  mpz_clear(B);
-  mpz_clear(sqrtn);
-  mpz_clear(m);
-  mpz_clear(f);
-  mpz_clear(t);
-  return success;
-}
-#endif
-
-int _GMP_primality_bls(mpz_t n, int do_quick)
-{
-  mpz_t nm1, A, B, t, m, f, r, s;
-  mpz_t mstack[PRIM_STACK_SIZE];
-  mpz_t fstack[PRIM_STACK_SIZE];
-  int msp = 0;
-  int fsp = 0;
-  int success = 1;
-
-  /* We need to do this for BLS */
-  if (mpz_even_p(n)) return -1;
-
-  mpz_init(nm1);
-  mpz_sub_ui(nm1, n, 1);
-  mpz_init_set_ui(A, 1);
-  mpz_init_set(B, nm1);
-  mpz_init(m);
-  mpz_init(f);
-  mpz_init(t);
-  mpz_init(r);
-  mpz_init(s);
-
-  { /* Pull small factors out */
-    PRIME_ITERATOR(iter);
-    UV tf;
-    for (tf = 2; tf < 2000; tf = prime_iterator_next(&iter)) {
-      if (mpz_cmp_ui(B, tf*tf) < 0) break;
-      if (mpz_divisible_ui_p(B, tf)) {
-        if (fsp >= PRIM_STACK_SIZE) { success = 0; break; }
-        mpz_init_set_ui(fstack[fsp++], tf);
-        do {
-          mpz_mul_ui(A, A, tf);
-          mpz_divexact_ui(B, B, tf);
-        } while (mpz_divisible_ui_p(B, tf));
-      }
-    }
-    prime_iterator_destroy(&iter);
-  }
-
-  if (success) {
-    mpz_set(f, B);
-    primality_handle_factor(f, _GMP_primality_bls, 1);
-  }
-
-  while (success) {
-    mpz_mul_ui(t, A, 2);
-    mpz_tdiv_q(s, B, t);
-    mpz_mod(r, B, t);
-
-    mpz_mul(m, t, A);    /* m = 2*A*A              */
-    mpz_sub_ui(t, r, 1); /* t = r-1                */
-    mpz_mul(t, t, A);    /* t = A*(r-1)            */
-    mpz_add(m, m, t);    /* m = 2A^2 + A(r-1)      */
-    mpz_add_ui(m, m, 1); /* m = 2A^2 + A(r-1) + 1  */
-    mpz_add_ui(t, A, 1);
-    mpz_mul(m, m, t);
-
-    if (mpz_cmp(m, n) > 0)
-      break;
-    success = 0;
-    /* If the stack is empty, we have failed. */
-    if (msp == 0)
-      break;
-    /* pop a component off the stack */
-    mpz_set(m, mstack[--msp]); mpz_clear(mstack[msp]);
-
-    /* Try to factor it without trying too hard */
-    if (!success)  success = _GMP_power_factor(m, f);
-
-    if (!success && mpz_cmp_ui(m, (unsigned long)(UV_MAX>>2)) < 0) {
-      UV ui_m = mpz_get_ui(m);
-      UV ui_factors[2];
-      if (!mpz_cmp_ui(m, ui_m)) {
-        success = racing_squfof_factor(ui_m, ui_factors, 200000)-1;
-        if (success)
-          mpz_set_ui(f, ui_factors[0]);
-      }
-    }
-    if (do_quick) {
-      /* These keep the time in the realm of 2ms per input number, which
-       * means they adjust for size.  Success varies by size:
-       *   99% of  70-bit inputs
-       *   90% of  90-bit inputs
-       *   58% of 120-bit inputs
-       *   25% of 160-bit inputs
-       *   10% of 190-bit inputs
-       */
-      UV log2m = mpz_sizeinbase(m, 2);
-      UV brent_rounds = (log2m <= 64) ? 100000 : 100000 / (log2m-63);
-      int final_B2 = 1000 * (150-(int)log2m);
-      if (log2m < 70) brent_rounds *= 3;
-      if (!success)  success = _GMP_pminus1_factor(m, f, 100, 1000);
-      if (!success)  success = _GMP_pminus1_factor(m, f, 1000, 10000);
-      if (!success && log2m < 80)  success = _GMP_ECM_FACTOR(m, f, 150, 5);
-      if (!success)  success = _GMP_pbrent_factor(m, f, 3, brent_rounds);
-      if (!success && final_B2 > 10000)  success = _GMP_pminus1_factor(m, f, 10000, final_B2);
-    } else {
-      if (!success)  success = _GMP_pminus1_factor(m, f, 100, 1000);
-      if (!success)  success = _GMP_pminus1_factor(m, f, 1000, 10000);
-      if (!success)  success = _GMP_pminus1_factor(m, f, 10000, 200000);
-      if (!success)  success = _GMP_ECM_FACTOR(m, f, 500, 30);
-      if (!success)  success = _GMP_ECM_FACTOR(m, f, 2000, 20);
-      if (!success)  success = _GMP_pminus1_factor(m, f, 200000, 4000000);
-      if (!success)  success = _GMP_ECM_FACTOR(m, f, 10000, 10);
-      /* QS.  Uses lots of memory, but finds multiple factors quickly */
-      if (!success && mpz_sizeinbase(m,10)>=30 && mpz_sizeinbase(m,2)<300) {
-            mpz_t farray[66];
-            int i, nfactors;
-            for (i = 0; i < 66; i++)
-              mpz_init(farray[i]);
-            nfactors = _GMP_simpqs(m, farray);
-            /* Insert all found factors */
-            if (nfactors > 1) {
-              success = 1;
-              for (i = 0; i < nfactors; i++) {
-                primality_handle_factor(farray[i], _GMP_primality_bls, 0);
-              }
-            }
-            for (i = 0; i < 66; i++)
-              mpz_clear(farray[i]);
-            if (success)
-              continue;
-      }
-      if (!success) {
-        UV i;
-        UV B1 = 10000;
-        UV curves = 10;
-        for (i = 1; i < 18; i++) {
-          B1 *= 2;
-          success = _GMP_ECM_FACTOR(m, f, B1, curves);
-          if (success) break;
-        }
-      }
-    }
-    /* If we couldn't factor m and the stack is empty, we've failed. */
-    if ( (!success) && (msp == 0) )
-      break;
-    /* Put the two factors f and m/f into the stacks, smallest first */
-    mpz_divexact(m, m, f);
-    if (mpz_cmp(m, f) < 0) { mpz_set(t, m); mpz_set(m, f); mpz_set(f, t); }
-    primality_handle_factor(f, _GMP_primality_bls, 0);
-    primality_handle_factor(m, _GMP_primality_bls, 0);
-  }
-
-  if (success) {
-    int pcount, a, proper_r;
-    int const alimit = do_quick ? 200 : 10000;
-    mpz_t p, ap;
-
-    mpz_init(p);
-    mpz_init(ap);
-
-    proper_r = 0;
-    if (!mpz_sgn(s)) {
-      proper_r = 1;
-    } else {
-      mpz_mul(t, r, r);
-      mpz_submul_ui(t, s, 8);   /* t = r^2 - 8s */
-      if ( ! mpz_perfect_square_p(t) )
-        proper_r = 1;
-    }
-
-    for (pcount = 0; success && pcount < fsp; pcount++) {
-      PRIME_ITERATOR(iter);
-      mpz_set(p, fstack[pcount]);
-      success = 0;
-      for (a = 2; !success && a <= alimit; a = prime_iterator_next(&iter)) {
-        mpz_set_ui(ap, a);
-        /* Does a^(n-1) % n = 1 ? */
-        mpz_powm(t, ap, nm1, n);
-        if (mpz_cmp_ui(t, 1) != 0)
-          continue;
-        /* Does gcd(a^((n-1)/f)-1,n) = 1 ? */
-        mpz_divexact(B, nm1, p);
-        mpz_powm(t, ap, B, n);
-        mpz_sub_ui(t, t, 1);
-        mpz_gcd(t, t, n);
-        if (mpz_cmp_ui(t, 1) != 0)
-          continue;
-        success = 1;   /* We found an a for this p */
-      }
-      prime_iterator_destroy(&iter);
-    }
-
-    /* If all cases hold then n is prime if and only if proper_r */
-    if ( success && !proper_r )
-      success = -1;
-    mpz_clear(p);
-    mpz_clear(ap);
-  }
-  while (msp-- > 0) {
-    mpz_clear(mstack[msp]);
-  }
-  while (fsp-- > 0) {
-    mpz_clear(fstack[fsp]);
-  }
-  mpz_clear(nm1);
-  mpz_clear(A);
-  mpz_clear(B);
-  mpz_clear(m);
-  mpz_clear(f);
-  mpz_clear(t);
-  mpz_clear(r);
-  mpz_clear(s);
-  return success;
-}
-
-
 /* Modifies argument */
 void _GMP_next_prime(mpz_t n)
 {
@@ -1202,7 +760,7 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
   mpz_init(savea);
   mpz_init(t);
 
-  if (_verbose>2) gmp_printf("# trying %Zd  with B=%lu\n", n, (unsigned long)B1);
+  if (_verbose>2) gmp_printf("# p-1 trying %Zd (B1=%lu B2=%lu)\n", n, (unsigned long)B1, (unsigned long)B2);
 
   /* STAGE 1
    * Montgomery 1987 p249-250 and Brent 1990 p5 both indicate we can calculate
@@ -1296,7 +854,6 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
     mpz_t precomp_bm[111];
     int   is_precomp[111] = {0};
 
-    if (_verbose>2) gmp_printf("# Starting second stage from %lu to %lu\n", (unsigned long)B1, (unsigned long)B2);
     mpz_init(bmdiff);
     mpz_init_set(bm, a);
     mpz_init_set_ui(b, 1);
@@ -1352,7 +909,6 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
       if (is_precomp[j])
         mpz_clear(precomp_bm[j]);
     }
-    if (_verbose>2) gmp_printf("# End second stage\n");
     if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) )
       goto end_success;
   }
@@ -1364,8 +920,11 @@ int _GMP_pminus1_factor(mpz_t n, mpz_t f, UV B1, UV B2)
     mpz_clear(a);
     mpz_clear(savea);
     mpz_clear(t);
-    if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) )
+    if ( (mpz_cmp_ui(f, 1) != 0) && (mpz_cmp(f, n) != 0) ) {
+      if (_verbose>2) gmp_printf("# p-1: %Zd\n", f);
       return 1;
+    }
+    if (_verbose>2) gmp_printf("# p-1: no factor\n");
     mpz_set(f, n);
     return 0;
 }
